-- **************************************************************************
-- *                        HUNTER STABLE INFO v4.2.1
-- *
-- *                 (c) 2011 by Rappi (Lordaeron - EU)
-- *
-- **************************************************************************


-- ********************************** Fonts *********************************
local titleFont = CreateFont("titleFont");
titleFont:SetTextColor(1,1,1);
titleFont:SetFont(GameTooltipText:GetFont(), 14);

local headerFont = CreateFont("headerFont");
headerFont:SetTextColor(1,0.82,0);
headerFont:SetFont(GameTooltipHeaderText:GetFont(), 12);

local standardFont = CreateFont("standardFont");
standardFont:SetTextColor(1,0.82,0);
standardFont:SetFont(GameTooltipText:GetFont(), 12);

local hintFont = CreateFont("hintFont");
hintFont:SetTextColor(0.77,0.64,0);
hintFont:SetFont(GameTooltipText:GetFont(), 8);

local pnFont = CreateFont("pnFont");
pnFont:SetTextColor(1,0.82,0);
pnFont:SetFont(GameTooltipText:GetFont(), 20, "OUTLINE");

-- ***************************** 3D Model Frame *****************************
local pf = CreateFrame("Frame","PetFrame",UIParent);

pf:SetAttribute("UIPanelLayout-defined", true);
pf:SetAttribute("UIPanelLayout-enabled", true);
pf:SetAttribute("UIPanelLayout-area", "left");
pf:SetAttribute("UIPanelLayout-pushable", 5);
pf:SetAttribute("UIPanelLayout-width", 256);
pf:SetAttribute("UIPanelLayout-whileDead", true);
pf:SetScale(1);
pf:SetSize(256, 256);
pf:SetMovable(true);
pf:EnableMouse(true);
pf:RegisterForDrag("LeftButton");
pf:SetScript("OnDragStart", pf.StartMoving);
pf:SetScript("OnDragStop", pf.StopMovingOrSizing);
pf:SetClampedToScreen(true);
pf:SetFrameStrata("HIGH");

-- Background Picture
local bg = pf:CreateTexture(nil,"BACKGROUND");
bg:SetTexture("Interface\\PetPaperDollFrame\\PetStatsBG-Hunter");
bg:SetDrawLayer("BACKGROUND");
bg:ClearAllPoints();
bg:SetPoint("BOTTOMLEFT", 2, 4);
bg:SetPoint("TOPRIGHT", -2, -4);
bg:SetTexCoord(0, .62, 0, .62);

-- Border
pf:SetBackdrop({
	tile = true; edgeSize = 8;
	edgeFile = "Interface\\BUTTONS\\UI-SliderBar-Border";
});

-- Title Background
local Titlebg = pf:CreateTexture(nil, "BORDER");
Titlebg:SetTexture("Interface\\AchievementFrame\\UI-Achievement-Title");
Titlebg:SetPoint("TOPRIGHT", -2, -3);
Titlebg:SetPoint("LEFT", 2, 0);
Titlebg:SetHeight(18);
Titlebg:SetTexCoord(0, 1, 0.3125, 0.6250);

-- Title
local Title = pf:CreateFontString(nil, "OVERLAY");
Title:SetFont(GameTooltipText:GetFont(), 14, "OUTLINE");
Title:SetTextColor(1, 0.82, 0);
Title:SetPoint("TOP", 0, -6);
Title:SetText("HunterStableInfo");

-- Pet Name
local pn = CreateFrame("Button", nil, pf);
pn:SetPoint("BOTTOM", 0, 26);
pn:SetSize(200, 22);
pn:SetNormalFontObject("pnFont");
pn:SetText("PetName");
pn:SetFrameStrata("DIALOG");
pn:Disable();

-- Close Button
local Close = CreateFrame("Button", nil, pf, "UIPanelCloseButton");
Close:SetPoint("TOPRIGHT");
Close:SetSize(32, 32);
Close:SetScale(0.8);
Close:SetHitRectInsets(8, 8, 8, 8);

-- 3D Model Frame
local Model = CreateFrame("PlayerModel", nil, pf);
Model:SetFrameStrata("HIGH");
Model:SetPoint("BOTTOM", 0, 6);
Model:SetSize(250, 228);
Model:SetCamera(1);
Model:SetFacing(math.pi/4);

-- Rotation Buttons
local btn_l = CreateFrame("Button", nil, Model);
btn_l:SetSize(22, 22);
btn_l:SetPoint("BOTTOM", -10, -2);
btn_l:SetNormalTexture("Interface\\Buttons\\UI-RotationLeft-Button-Up");
btn_l:SetPushedTexture("Interface\\Buttons\\UI-RotationLeft-Button-Down");
btn_l:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Round", "ADD");

btn_l:SetScript("OnMouseDown", function() Model:SetScript("OnUpdate",	function() Model:SetRotation(Model:GetFacing()+0.07); end); end);
btn_l:SetScript("OnMouseUp", function() Model:SetScript("OnUpdate",	nil); end);

local btn_r = CreateFrame("Button", nil, Model);
btn_r:SetSize(22, 22);
btn_r:SetPoint("BOTTOM", 10, -2);
btn_r:EnableMouse(true);
btn_r:SetNormalTexture("Interface\\Buttons\\UI-RotationRight-Button-Up");
btn_r:SetPushedTexture("Interface\\Buttons\\UI-RotationRight-Button-Down");
btn_r:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Round", "ADD");

btn_r:SetScript("OnMouseDown", function() Model:SetScript("OnUpdate",	function() Model:SetRotation(Model:GetFacing()-0.07); end); end);
btn_r:SetScript("OnMouseUp", function() Model:SetScript("OnUpdate",	nil); end);


HideUIPanel(pf);


-- ******************************** Variables *******************************
local tooltip;
local HSI_ANCHOR;

local frame = CreateFrame("Frame");
frame:RegisterEvent("ADDON_LOADED");
frame:RegisterEvent("PLAYER_ENTERING_WORLD");
frame:RegisterEvent("UNIT_PET");
frame:RegisterEvent("PET_STABLE_UPDATE");
frame:RegisterEvent("UNIT_NAME_UPDATE");
frame:RegisterEvent("UNIT_POWER");

local LibQTip = LibStub('LibQTip-1.0');
local LDB = LibStub("LibDataBroker-1.1");

local dataobj = LDB:NewDataObject("HunterStableInfo", {
	type = "data source",
	label = "HunterStableInfo",
	text = "",
	icon = "Interface\\Icons\\Ability_Hunter_BeastCall02",
});


function eventHandler(self, event, ...)
	if(event == "ADDON_LOADED") then
		if(show_stabledPets == nil) then
			show_stabledPets = true;
		end
		if(show_hints == nil) then
			show_hints = true;
		end
	end
	if(event == "PET_STABLE_UPDATE" or event == "UNIT_PET" or event == "PLAYER_ENTERING_WORLD") then
		dataobj.icon = frame:GetCurrentPetIcon();
		dataobj.text = frame:HunterStableInfo_GetButtonText();
	end
	if(event == "UNIT_NAME_UPDATE") then
		local arg1 = select(1, ...);
		if(arg1 == "pet") then
			dataobj.icon = frame:GetCurrentPetIcon();
			dataobj.text = frame:HunterStableInfo_GetButtonText();
		end
	end
	if(event == "UNIT_POWER") then
		local arg1, arg2 = select(1, ...);
		if(arg1 == "pet" and arg2 == "HAPPINESS") then
			dataobj.icon = frame:GetCurrentPetIcon();
			dataobj.text = frame:HunterStableInfo_GetButtonText();
		end
	end
end


frame:SetScript("OnEvent", eventHandler);

-- ******************************** Functions *******************************


-- **********
-- * Button *
-- **********
function frame:HunterStableInfo_GetButtonText()
	return frame:GetCurrentPetHappiness()..frame:GetCurrentPet();
end

function dataobj.OnClick(self, button)
	if(button == "LeftButton") then
		if(IsShiftKeyDown()) then
			ToggleCharacter("PetPaperDollFrame")
		else
			if(show_petAbilities) then
				show_petAbilities = false;
			else
				show_petAbilities = true;
			end
			dataobj.OnEnter(HSI_ANCHOR);
		end
	end
	if(button == "RightButton") then
		if(IsShiftKeyDown()) then
			if(show_stabledPets) then
				show_stabledPets = false;
			else
				show_stabledPets = true;
			end
		else
			if(show_hints) then
				show_hints = false;
			else
				show_hints = true;
			end
		end
		dataobj.OnEnter(HSI_ANCHOR);
	end
end

function dataobj.OnLeave(self, ...)
	if(show_petAbilities) then
		show_petAbilities = false;
	end
end


-- ***********
-- * Tooltip *
-- ***********
function dataobj.OnEnter(self)
	HSI_ANCHOR = self;
	if LibQTip:IsAcquired("HunterStableInfo") then
		tooltip:Clear();
		if(show_petAbilities) then
			tooltip:SetColumnLayout(6, "LEFT", "CENTER", "LEFT", "LEFT", "LEFT", "LEFT");
		else
			tooltip:SetColumnLayout(5, "RIGHT", "LEFT", "CENTER", "LEFT", "CENTER");
		end
	else
		tooltip = LibQTip:Acquire("HunterStableInfo", 5, "RIGHT", "LEFT", "CENTER", "LEFT", "CENTER");
		tooltip:SetHeaderFont(headerFont);
		tooltip:SetFont(standardFont);

		tooltip:SmartAnchorTo(self);
		tooltip:SetAutoHideDelay(0.1, self);
	end
	
	-- Title
	local line = tooltip:AddLine();
	tooltip:SetCell(line, 1, "HunterStableInfo", titleFont, "CENTER", 0);
	tooltip:AddLine(" ");
	
	-- Hunter Class Check
	local localizedClass, englishClass = UnitClass("player");
	if(englishClass ~= "HUNTER") then
		tooltip:AddLine(HUNTERSTABLEINFO_NOTAHUNTER);
	else
	-- TOOLTIP: Pet Abilities
		if(show_petAbilities) then
			-- Table Head
			line = tooltip:AddHeader();
			tooltip:SetCell(line, 1, HUNTERSTABLEINFO_FAMILY);
			line = tooltip:SetCell(line, 2, HUNTERSTABLEINFO_CLASS);
			line = tooltip:SetCell(line, 3, HUNTERSTABLEINFO_ABILITIESTITLE);
			tooltip:AddSeparator();
			
			-- Sorting function
			local function pairsByKeys(t, f)
				local a = {}
					for n in pairs(t) do table.insert(a, n) end
					table.sort(a, f)
					local i = 0      -- iterator variable
					local iter = function ()   -- iterator function
						i = i + 1
						if a[i] == nil then return nil
						else return a[i], t[a[i]]
						end
					end
				return iter
			end
			
			for i,v in pairsByKeys(HUNTERSTABLEINFO_petAbilities) do
				line = tooltip:AddLine();
				if(i == HUNTERSTABLEINFO_Chimaera or i == HUNTERSTABLEINFO_CoreHound or i == HUNTERSTABLEINFO_Devilsaur or i == HUNTERSTABLEINFO_Rhino or i == HUNTERSTABLEINFO_ShaleSpider or i == HUNTERSTABLEINFO_Silithid or i == HUNTERSTABLEINFO_SpiritBeast or i == HUNTERSTABLEINFO_Worm) then
					tooltip:SetCell(line, 1, "|T"..HUNTERSTABLEINFO_petIcons[i]..":16:16:0:0|t |cff33c233"..i.." |cff119011"..HUNTERSTABLEINFO_EXOTIC.."|r");
				else
					tooltip:SetCell(line, 1, "|T"..HUNTERSTABLEINFO_petIcons[i]..":16:16:0:0|t |cff33c233"..i.."|r");
				end
				if(HUNTERSTABLEINFO_petClass[i] == 1) then tooltip:SetCell(line, 2, "|cfffff555"..HUNTERSTABLEINFO_FEROCITY.."|r"); end
				if(HUNTERSTABLEINFO_petClass[i] == 2) then tooltip:SetCell(line, 2, "|cff55f5ff"..HUNTERSTABLEINFO_CUNNING.."|r"); end
				if(HUNTERSTABLEINFO_petClass[i] == 3) then tooltip:SetCell(line, 2, "|cffff5533"..HUNTERSTABLEINFO_TENACITY.."|r"); end
				tooltip:SetCell(line, 3, GetSpellLink(HUNTERSTABLEINFO_petAbilities[i][1]));
				tooltip:SetCellScript(line, 3, "OnEnter", function()
																											GameTooltip:SetOwner(UIParent, "ANCHOR_CURSOR");
																											GameTooltip:SetHyperlink("spell:"..HUNTERSTABLEINFO_petAbilities[i][1]);
																											GameTooltip:Show();
																											tooltip:SetFrameStrata("FULLSCREEN_DIALOG");
																										end);
				tooltip:SetCellScript(line, 3, "OnLeave", function() GameTooltip:Hide(); tooltip:SetFrameStrata("TOOLTIP"); end);
				if(HUNTERSTABLEINFO_petAbilities[i][2]) then
					tooltip:SetCell(line, 4, GetSpellLink(HUNTERSTABLEINFO_petAbilities[i][2]),standardFont,"LEFT");
					tooltip:SetCellScript(line, 4, "OnEnter", function()
																											GameTooltip:SetOwner(UIParent, "ANCHOR_CURSOR");
																											GameTooltip:SetHyperlink("spell:"..HUNTERSTABLEINFO_petAbilities[i][2]);
																											GameTooltip:Show();
																											tooltip:SetFrameStrata("FULLSCREEN_DIALOG");
																										end);
					tooltip:SetCellScript(line, 4, "OnLeave", function() GameTooltip:Hide(); tooltip:SetFrameStrata("TOOLTIP"); end);
				end
				if(HUNTERSTABLEINFO_petAbilities[i][3]) then
					tooltip:SetCell(line, 5, GetSpellLink(HUNTERSTABLEINFO_petAbilities[i][3]));
					tooltip:SetCellScript(line, 5, "OnEnter", function()
																											GameTooltip:SetOwner(UIParent, "ANCHOR_CURSOR");
																											GameTooltip:SetHyperlink("spell:"..HUNTERSTABLEINFO_petAbilities[i][3]);
																											GameTooltip:Show();
																											tooltip:SetFrameStrata("FULLSCREEN_DIALOG");
																										end);
					tooltip:SetCellScript(line, 5, "OnLeave", function() GameTooltip:Hide(); tooltip:SetFrameStrata("TOOLTIP"); end);
				end
				if(HUNTERSTABLEINFO_petAbilities[i][4]) then
					tooltip:SetCell(line, 6, GetSpellLink(HUNTERSTABLEINFO_petAbilities[i][4]));
					tooltip:SetCellScript(line, 6, "OnEnter", function()
																											GameTooltip:SetOwner(UIParent, "ANCHOR_CURSOR");
																											GameTooltip:SetHyperlink("spell:"..HUNTERSTABLEINFO_petAbilities[i][4]);
																											GameTooltip:Show();
																											tooltip:SetFrameStrata("FULLSCREEN_DIALOG");
																										end);
					tooltip:SetCellScript(line, 6, "OnLeave", function() GameTooltip:Hide(); tooltip:SetFrameStrata("TOOLTIP"); end);
				end
				tooltip:SetLineScript(line, "OnMouseUp", PetAbilities_OnMouseUp, i);
			end
		else
		-- TOOLTIP: Pet Stable
			-- Table Head
			line = tooltip:AddHeader();
			tooltip:SetCell(line, 1, " ");
			line = tooltip:SetCell(line, 2, HUNTERSTABLEINFO_NAME);
			line = tooltip:SetCell(line, 3, HUNTERSTABLEINFO_LEVEL);
			line = tooltip:SetCell(line, 4, HUNTERSTABLEINFO_FAMILY);
			line = tooltip:SetCell(line, 5, HUNTERSTABLEINFO_CLASS);
			tooltip:AddSeparator();
			
			-- Active Pets
			local loyaltyColor = "";
			for i=1, 5 do
				local petIcon, petName, petLevel, petType, petLoyalty = GetStablePetInfo(i, true);
				if(petLoyalty == HUNTERSTABLEINFO_FEROCITY) then loyaltyColor = "|cfffff555"; end
				if(petLoyalty == HUNTERSTABLEINFO_CUNNING) then loyaltyColor = "|cff55f5ff"; end
				if(petLoyalty == HUNTERSTABLEINFO_TENACITY) then loyaltyColor = "|cffff5533"; end
				
				line = tooltip:AddLine();
				if(petName ~= nil) then
					tooltip:SetCell(line, 1, i);
					tooltip:SetCell(line, 2, "|cfffafafa"..petName.."|r");
					tooltip:SetCell(line, 3, "|cffbbbbbb"..petLevel.."|r");
					if(petType == HUNTERSTABLEINFO_Chimaera or petType == HUNTERSTABLEINFO_CoreHound or petType == HUNTERSTABLEINFO_Devilsaur or petType == HUNTERSTABLEINFO_Rhino or petType == HUNTERSTABLEINFO_ShaleSpider or petType == HUNTERSTABLEINFO_Silithid or petType == HUNTERSTABLEINFO_SpiritBeast or petType == HUNTERSTABLEINFO_Worm) then
						tooltip:SetCell(line, 4, "|T"..petIcon..":16:16:0:0|t |cff33c233"..petType.." |cff119011"..HUNTERSTABLEINFO_EXOTIC.."|r");
					else
						tooltip:SetCell(line, 4, "|T"..petIcon..":16:16:0:0|t |cff33c233"..petType.."|r");
					end
					tooltip:SetCell(line, 5, ""..loyaltyColor..petLoyalty.."|r");
				else
					tooltip:SetCell(line, 1, i);
					tooltip:SetCell(line, 2, "|cff7d7d7d"..HUNTERSTABLEINFO_EMPTYSLOT.."|r");
				end
				if(petName ~= nil) then
					tooltip:SetLineScript(line, "OnMouseUp", Pet_OnMouseUp, string.format("%s:%s:%s:%s", petName, petType, petIcon, i));
				end
			end
			
			-- Stabled Pets
			if(show_stabledPets) then
				tooltip:AddSeparator();
				for i=6, 25 do
					local petIcon, petName, petLevel, petType, petLoyalty = GetStablePetInfo(i, true);
					if(petLoyalty == HUNTERSTABLEINFO_FEROCITY) then loyaltyColor = "|cfffff555"; end
					if(petLoyalty == HUNTERSTABLEINFO_CUNNING) then loyaltyColor = "|cff55f5ff"; end
					if(petLoyalty == HUNTERSTABLEINFO_TENACITY) then loyaltyColor = "|cffff5533"; end
					
					line = tooltip:AddLine();
					if(petName ~= nil) then
						tooltip:SetCell(line, 1, i);
						tooltip:SetCell(line, 2, "|cfffafafa"..petName.."|r");
						tooltip:SetCell(line, 3, "|cffbbbbbb"..petLevel.."|r");
						if(petType == HUNTERSTABLEINFO_Chimaera or petType == HUNTERSTABLEINFO_CoreHound or petType == HUNTERSTABLEINFO_Devilsaur or petType == HUNTERSTABLEINFO_Rhino or petType == HUNTERSTABLEINFO_ShaleSpider or petType == HUNTERSTABLEINFO_Silithid or petType == HUNTERSTABLEINFO_SpiritBeast or petType == HUNTERSTABLEINFO_Worm) then
							tooltip:SetCell(line, 4, "|T"..petIcon..":16:16:0:0|t |cff33c233"..petType.." |cff119011"..HUNTERSTABLEINFO_EXOTIC.."|r");
						else
							tooltip:SetCell(line, 4, "|T"..petIcon..":16:16:0:0|t |cff33c233"..petType.."|r");
						end
						tooltip:SetCell(line, 5, ""..loyaltyColor..petLoyalty.."|r");
					else
						tooltip:SetCell(line, 1, i);
						tooltip:SetCell(line, 2, "|cff7d7d7d"..HUNTERSTABLEINFO_EMPTYSLOT.."|r");
					end
					if(petName ~= nil) then
						tooltip:SetLineScript(line, "OnMouseUp", Pet_OnMouseUp, string.format("%s:%s:%s:%s", petName, petType, petIcon, i));
					end
				end
			end
			
			-- Show number of pets in stable
			tooltip:AddSeparator();
			line = tooltip:AddLine();
			tooltip:SetCell(line, 1, frame:CountPets()..HUNTERSTABLEINFO_MAXSTABLE, standardFont, "CENTER", 0);
			
		end
		-- Hints
		if(show_hints) then
			tooltip:AddLine(" ");
			line = tooltip:AddLine();
			tooltip:SetCell(line, 1, HUNTERSTABLEINFO_HINT1, hintFont, "LEFT", 0);
			line = tooltip:AddLine();
			tooltip:SetCell(line, 1, HUNTERSTABLEINFO_HINT2, hintFont, "LEFT", 0);
			line = tooltip:AddLine();
			tooltip:SetCell(line, 1, HUNTERSTABLEINFO_HINT3, hintFont, "LEFT", 0);
			line = tooltip:AddLine();
			tooltip:SetCell(line, 1, HUNTERSTABLEINFO_HINT4, hintFont, "LEFT", 0);
			line = tooltip:AddLine();
			tooltip:SetCell(line, 1, HUNTERSTABLEINFO_HINT5, hintFont, "LEFT", 0);
			line = tooltip:AddLine();
			tooltip:SetCell(line, 1, HUNTERSTABLEINFO_HINT6, hintFont, "LEFT", 0);
			line = tooltip:AddLine();
			tooltip:SetCell(line, 1, HUNTERSTABLEINFO_HINT7, hintFont, "LEFT", 0);
		end
	end
	
	tooltip:UpdateScrolling();
	tooltip:Show();
end


-- ************************
-- * Some other functions *
-- ************************
function frame:GetCurrentPetName()
	if(UnitName("pet")) then
		return UnitName("pet")
	else
		return HUNTERSTABLEINFO_NOPET
	end
end
function frame:GetCurrentPetLevel()
	if(UnitLevel("pet") ~= 0) then
		return "|r (|cffbbbbbb"..UnitLevel("pet").."|r)"
	else
		return ""
	end
end
function frame:GetCurrentPetType()
	if(UnitCreatureFamily("pet")) then
		return "|r - |cff33c233"..UnitCreatureFamily("pet")
	else
		return ""
	end
end
function frame:GetCurrentPetIcon()
	if(GetPetIcon()) then
		return GetPetIcon()
	else
		return "Interface\\Icons\\Ability_Hunter_BeastCall02"
	end
end
function frame:GetCurrentPetHappiness()
	local tmp = GetPetHappiness();
	if(tmp == 1) then return "|cffff0000"; end
	if(tmp == 2) then return "|cffffff00"; end
	if(tmp == 3) then return "|cff00ff00"; end
	if(tmp == nil) then return "|cff9d9d9d"; end
end
function frame:GetCurrentPet()
	return frame:GetCurrentPetName()..frame:GetCurrentPetLevel()
end
function frame:CountPets()
	local count = 0;
	for i=1, 25 do
		local x = GetStablePetInfo(i, true);
		if(x ~= nil) then
			count = count + 1;
		end
	end
	return count
end


-- *************************
-- * Print Pet Information *
-- *************************
function Pet_OnMouseUp(cell, arg, button)
	local petName, petType, petIcon, stableID = string.split(":", arg);
	if(button == "LeftButton") then
		pn:SetText(petName);
		SetPetStablePaperdoll(Model, stableID);
		Model:SetFacing(math.pi/4);
		ShowUIPanel(pf);
	end
	if(button == "RightButton") then
		if(IsShiftKeyDown()) then
			print("|cffABD473<HunterStableInfo> |T"..petIcon..":16:16:0:0|t |cffffd200"..petName.."|r "..HUNTERSTABLEINFO_EATS.." "..GetPetDiet(stableID)..".");
		else
			print("|cffABD473<HunterStableInfo> |T"..petIcon..":16:16:0:0|t |cffffd200"..petName.."|r's "..HUNTERSTABLEINFO_ABILITIES.." "..GetPetAbilities(petType)..".");
		end
	end
end

function PetAbilities_OnMouseUp(cell, arg, button)
	if(button == "LeftButton") then
		if(GetLocale() == "deDE") then
			print("|cffABD473<HunterStableInfo>|r Ein(e) |T"..HUNTERSTABLEINFO_petIcons[arg]..":16:16:0:0|t |cffffd200"..arg.."|r hat die F\195\164higkeiten: "..GetPetAbilities(arg)..".");
		else
			print("|cffABD473<HunterStableInfo>|r Abilities of a |T"..HUNTERSTABLEINFO_petIcons[arg]..":16:16:0:0|t |cffffd200"..arg.."|r are: "..GetPetAbilities(arg)..".");
		end
	end
end

function GetPetAbilities(petType)
	local tmp = "";
	for k,v in pairs(HUNTERSTABLEINFO_petAbilities[petType]) do
		local link = GetSpellLink(v),_;
		tmp = tmp .. link..", ";
	end
	tmp = string.sub(tmp, 1, -3);
	return tmp
end

function GetPetDiet(stableID)
	local diet = { GetStablePetFoodTypes(stableID) };
	local tmp = "";
	for k,v in ipairs(diet) do
		tmp = tmp .. v..", ";
	end
	tmp = string.sub(tmp, 1, -3);
	return tmp
end