-- In game configuration, using Blizzard addon frame.

local RefreshConfigurationSettingsBM
local RefreshConfigurationSettingsMM
local RefreshConfigurationSettingsSV
local AddTimer
local AddTimerFrame
local ConfirmDelete
local CreateButton
local deleteFrame
local CreateDropdown
local info = { }


local function Frame_OnEnter(self)
	if self.desc then
		GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
		GameTooltip:SetText(self.desc, nil, nil, nil, nil, true)
	end
end


local function Button_OnEnter(self)
	return Frame_OnEnter(self:GetParent():GetParent())
end


local function Frame_OnLeave()
	GameTooltip:Hide()
end


local function OnClick(self)
	PlaySound("igMainMenuOptionCheckBoxOn")
	ToggleDropDownMenu(nil, nil, self:GetParent())
end


local function OnHide()
	CloseDropDownMenus()
end


local function GetValue(self)
	return UIDropDownMenu_GetSelectedValue(self.dropdown) or self.valueText:GetText()
end


local function SetValue(self, value, text)
	self.valueText:SetText(text or value)
	UIDropDownMenu_SetSelectedValue(self.dropdown, value or "UNKNOWN")
end


local function CancelConfigurationSettings()
	JSHB.setupOptionsPane()
end


local function ApplyConfigurationSettings()
	JSHB.updateSettings()
end


ConfirmDelete = function(spec, parent, timer)

	if AddTimerFrame then AddTimerFrame:Hide() end

	local specTimers = (spec == "bm") and JSHB.options.bm.Timers or ((spec == "mm") and JSHB.options.mm.Timers or JSHB.options.sv.Timers)
	local specName = (spec == "bm") and JSHB.locale.namebm or ((spec == "mm") and JSHB.locale.namemm or JSHB.locale.namesv)

	StaticPopupDialogs["JSHB.optionsDeleteConfirmation"] = {
		text = strupper(
			select(1, GetSpellInfo(specTimers[timer][1])).."\n\n|cffff0000"..JSHB.locale.confirmdelete1..specName..JSHB.locale.confirmdelete2.."|r"),
		button1 = "Delete",
		button2 = "Cancel",
		OnAccept = function()
			StaticPopup_Hide("JSHB.optionsDeleteConfirmation")
			table.remove(specTimers, timer)
			if spec == "bm" then
				RefreshConfigurationSettingsBM()
			elseif spec == "mm" then
				RefreshConfigurationSettingsMM()
			else
				RefreshConfigurationSettingsSV()
			end
		end,
		cancels = "JSHB.optionsDeleteConfirmation",
		exclusive = true,
		showAlert = true,
		timeout = 0,
		whileDead = true,
		hideOnEscape = true,
	}

	StaticPopup_Show("JSHB.optionsDeleteConfirmation")
end


local function createTimerFrame(spec, parent, frameName, timer, anchorFrame, offsetY, spaceY)

	local specTimers = (spec == "bm") and JSHB.options.bm.Timers or ((spec == "mm") and JSHB.options.mm.Timers or JSHB.options.sv.Timers)

	local frame = CreateFrame("Frame", frameName, parent)
	frame:SetHeight(32)
	frame:SetWidth(360)
	frame:SetPoint("TOPLEFT", anchorFrame, "BOTTOMLEFT", 0, offsetY - (spaceY or 5))
	frame:Show()

	local bgtex = frame:CreateTexture(nil, "BACKGROUND", frame)
	bgtex:SetAllPoints(frame)
	bgtex:SetTexture(0, 0, 0, 1)
	
	frame.border = CreateFrame("Frame", nil, frame)
	frame.border:SetBackdrop({
	  bgFile = "",
	  edgeFile = JSHB.tex[JSHB.tex.getTextureIndex("Blank")][2], 
	  tile = false, tileSize = 0, edgeSize = 1, 
	  insets = { left = -1, right = -1, top = -1, bottom = -1}
	})
	frame.border:SetBackdropColor(.1,.1,.1,1)
	frame.border:SetBackdropBorderColor(.6,.6,.6,1)
	frame.border:SetPoint("TOPLEFT", -1, 1)
	frame.border:SetPoint("BOTTOMRIGHT", 1, -1)

	frame.deletebutton = CreateFrame("Button", nil, frame)
	frame.deletebutton.desc = JSHB.locale.deletebutton
	frame.deletebutton:SetPoint("TOPRIGHT", frame, "TOPRIGHT", -2, -6)
	frame.deletebutton.Texture = frame.deletebutton:CreateTexture(nil, "OVERLAY")
	frame.deletebutton.Texture:SetAllPoints(frame.deletebutton)
	frame.deletebutton.Texture:SetTexture("Interface\\Buttons\\UI-GroupLoot-Pass-Up")
	frame.deletebutton.Texture:SetTexCoord(0.08,0.92,0.08,0.92)
	frame.deletebutton:SetHeight(20)
	frame.deletebutton:SetWidth(20)
	frame.deletebutton:Enable()
	frame.deletebutton:EnableMouse()
	frame.deletebutton:SetScript("OnClick", function() ConfirmDelete(spec, parent, timer) end)
	frame.deletebutton:SetScript("OnEnter", Frame_OnEnter)
	frame.deletebutton:SetScript("OnLeave", Frame_OnLeave)

	frame.editbutton = CreateFrame("Button", nil, frame)
	frame.editbutton.desc = JSHB.locale.editbutton
	frame.editbutton:SetPoint("TOPRIGHT", frame.deletebutton, "TOPLEFT", -4, 0)
	frame.editbutton.Texture = frame.editbutton:CreateTexture(nil, "OVERLAY")
	frame.editbutton.Texture:SetAllPoints(frame.editbutton)
	frame.editbutton.Texture:SetTexture("Interface\\Icons\\Trade_Engineering")
	frame.editbutton.Texture:SetTexCoord(0.08,0.92,0.08,0.92)
	frame.editbutton:SetHeight(20)
	frame.editbutton:SetWidth(20)
	frame.editbutton:Enable()
	frame.editbutton:EnableMouse()
	frame.editbutton:SetScript("OnClick", function() AddTimer(spec, parent, timer) end)
	frame.editbutton:SetScript("OnEnter", Frame_OnEnter)
	frame.editbutton:SetScript("OnLeave", Frame_OnLeave)

	frame.icon = CreateFrame("Frame", nil, frame)
	frame.icon.desc = select(1, GetSpellInfo(specTimers[timer][1]))
	frame.icon:SetPoint("LEFT", frame, "LEFT", 4, 0)
	frame.icon.Texture = frame.icon:CreateTexture(nil, "OVERLAY")
	frame.icon.Texture:SetAllPoints(frame.icon)
	frame.icon.Texture:SetTexture(select(3, GetSpellInfo(specTimers[timer][1])))
	frame.icon:SetHeight(20)
	frame.icon:SetWidth(20)
	frame.icon:SetScript("OnEnter", Frame_OnEnter)
	frame.icon:SetScript("OnLeave", Frame_OnLeave)

	frame.text1 = frame:CreateFontString(frameName.."text1", "ARTWORK", frame)
	frame.text1:SetFontObject(GameFontNormal)	
	frame.text1:SetJustifyH("LEFT")
	frame.text1:SetWidth(108)
	frame.text1:SetPoint("LEFT", frame.icon, "RIGHT", 6, 0)
	frame.text1:SetText(select(1, GetSpellInfo(specTimers[timer][1])))

	frame.text2 = frame:CreateFontString(frameName.."text2", "ARTWORK", frame)
	frame.text2:SetFontObject(GameFontNormal)	
	frame.text2:SetJustifyH("LEFT")
	frame.text2:SetWidth(170)
	frame.text2:SetPoint("RIGHT", frame.editbutton, "LEFT", 6, 0)
	frame.text2:SetText((specTimers[timer][2] and JSHB.locale.spelltextdurfor or JSHB.locale.spelltextcdfor) .. 
				(specTimers[timer][3] and JSHB.locale.spelltextplayer or JSHB.locale.spelltexttarget) .. " - " .. 
				JSHB.timerpositions.getValueName(specTimers[timer][4]))
end


AddTimer = function(spec, parent, editTimer)
	if AddTimerFrame then AddTimerFrame:Hide() end

	local specTimers = (spec == "bm") and JSHB.options.bm.Timers or ((spec == "mm") and JSHB.options.mm.Timers or JSHB.options.sv.Timers)
	
	local spellTable = {}
	local x, y

	for x,y in pairs(JSHB.spellid) do
		spellTable[#spellTable + 1] = { select(1, GetSpellInfo(y)), y }
	end

	table.sort(spellTable, function(a,b) return a[1] < b[1] end)

	AddTimerFrame = CreateFrame("Frame", "JSBOptions_AddTimerFrame", parent)
	AddTimerFrame:SetHeight(300)
	AddTimerFrame:SetWidth(260)
	AddTimerFrame:SetPoint("CENTER", 0, 0)
	AddTimerFrame:Raise()
	AddTimerFrame:Show()
	AddTimerFrame:SetScript("OnHide", function() 
		AddTimerFrame:Hide()
		AddTimerFrame = nil
	end)
	AddTimerFrame.editTimer = editTimer
	AddTimerFrame.spec = spec
	AddTimerFrame.specTimers = specTimers
	AddTimerFrame.spellTable = spellTable

	local bgtex = AddTimerFrame:CreateTexture(nil, "BACKGROUND", AddTimerFrame)
	bgtex:SetAllPoints(AddTimerFrame)
	bgtex:SetTexture(0, 0, 0, 1)
	
	AddTimerFrame.border = CreateFrame("Frame", nil, AddTimerFrame)
	AddTimerFrame.border:SetBackdrop({
	  bgFile = "",
	  edgeFile = JSHB.tex[JSHB.tex.getTextureIndex("Blank")][2], 
	  tile = false, tileSize = 0, edgeSize = 1, 
	  insets = { left = -1, right = -1, top = -1, bottom = -1}
	})
	AddTimerFrame.border:SetBackdropColor(.1,.1,.1,1)
	AddTimerFrame.border:SetBackdropBorderColor(.6,.6,.6,1)
	AddTimerFrame.border:SetPoint("TOPLEFT", -1, 1)
	AddTimerFrame.border:SetPoint("BOTTOMRIGHT", 1, -1)

	local tableDurCD = { { JSHB.locale.cd, 0 }, { JSHB.locale.dur, 1 } }

	AddTimerFrame.pickDurOrCD = CreateDropdown(AddTimerFrame, JSHB.locale.durorcd, function()
		local i
		for i=1,#tableDurCD do
			info.text = tableDurCD[i][1]
			info.value = tableDurCD[i][2]
			info.func = function(self) AddTimerFrame.pickDurOrCD:SetValue(self.value, self.text) end
			info.checked = false
			UIDropDownMenu_AddButton(info)
		end
	end)

	AddTimerFrame.pickDurOrCD:SetPoint("TOPLEFT", AddTimerFrame, "TOPLEFT", 40, -80)
	AddTimerFrame.pickDurOrCD:SetPoint("TOPRIGHT", AddTimerFrame, "TOPRIGHT", -40, -110)

	UIDropDownMenu_SetSelectedValue(AddTimerFrame.pickDurOrCD.dropdown, 
		editTimer ~= nil and (specTimers[editTimer][2] == true and 1 or 0) or tableDurCD[1][2])
	AddTimerFrame.pickDurOrCD.valueText:SetText(
		editTimer ~= nil and (specTimers[editTimer][2]==true and tableDurCD[2][1] or tableDurCD[1][1]) or tableDurCD[1][1])

	local tablePlayerOrTarget = { { JSHB.locale.spelltextplayer, 1 }, { JSHB.locale.spelltexttarget, 0 } }
	AddTimerFrame.pickPlayerOrTarget = CreateDropdown(AddTimerFrame, JSHB.locale.pickplayerortarget, function()
		local i
		for i=1,#tablePlayerOrTarget do
			info.text = tablePlayerOrTarget[i][1]
			info.value = tablePlayerOrTarget[i][2]
			info.func = function(self) AddTimerFrame.pickPlayerOrTarget:SetValue(self.value, self.text) end
			info.checked = false
			UIDropDownMenu_AddButton(info)
		end
	end)
	AddTimerFrame.pickPlayerOrTarget:SetPoint("TOPLEFT", AddTimerFrame.pickDurOrCD, "TOPLEFT", 0, -55)
	AddTimerFrame.pickPlayerOrTarget:SetPoint("TOPRIGHT", AddTimerFrame.pickDurOrCD, "TOPRIGHT", 0, -55)

	UIDropDownMenu_SetSelectedValue(AddTimerFrame.pickPlayerOrTarget.dropdown,
		editTimer ~= nil and (specTimers[editTimer][3]==false and 0 or 1) or tablePlayerOrTarget[1][2])
	AddTimerFrame.pickPlayerOrTarget.valueText:SetText(
		editTimer ~= nil and (specTimers[editTimer][3]==true and tablePlayerOrTarget[1][1] or tablePlayerOrTarget[2][1]) or tablePlayerOrTarget[1][1])

	AddTimerFrame.pickLocation = CreateDropdown(AddTimerFrame, JSHB.locale.pickLocation, function()
		local i
		for i=1,#JSHB.timerpositions do
			info.text = JSHB.timerpositions[i][1]
			info.value = JSHB.timerpositions[i][2]
			info.func = function(self) AddTimerFrame.pickLocation:SetValue(self.value, self.text) end
			info.checked = false
			UIDropDownMenu_AddButton(info)
		end
	end)
	AddTimerFrame.pickLocation:SetPoint("TOPLEFT", AddTimerFrame.pickPlayerOrTarget, "TOPLEFT", 0, -55)
	AddTimerFrame.pickLocation:SetPoint("TOPRIGHT", AddTimerFrame.pickPlayerOrTarget, "TOPRIGHT", 0, -55)
	UIDropDownMenu_SetSelectedValue(AddTimerFrame.pickLocation.dropdown, editTimer ~= nil and specTimers[editTimer][4] or JSHB.timerpositions[1][2])
	AddTimerFrame.pickLocation.valueText:SetText(
		editTimer ~= nil and JSHB.timerpositions.getValueName(specTimers[editTimer][4]) or JSHB.timerpositions[1][1])

	AddTimerFrame.pickSpell = CreateScrollingDropdown(AddTimerFrame, JSHB.locale.pickSpell, spellTable)
	AddTimerFrame.pickSpell:SetPoint("TOPLEFT", AddTimerFrame, "TOPLEFT", 40, -25)
	AddTimerFrame.pickSpell:SetValue(editTimer ~= nil and select(1, GetSpellInfo(specTimers[editTimer][1])) or spellTable[1][1])
	local button_OnClick = AddTimerFrame.pickSpell.button:GetScript("OnClick")
	AddTimerFrame.pickSpell.button:SetScript("OnClick", function(self)
		button_OnClick(self)
		AddTimerFrame.pickSpell.dropdown.list:Hide()

		local OnShow = AddTimerFrame.pickSpell.dropdown.list:GetScript("OnShow")
		AddTimerFrame.pickSpell.dropdown.list:SetScript("OnShow", function(self)
			OnShow(self)
		end)

		local OnVerticalScroll = AddTimerFrame.pickSpell.dropdown.list.scrollFrame:GetScript("OnVerticalScroll")
		AddTimerFrame.pickSpell.dropdown.list.scrollFrame:SetScript("OnVerticalScroll", function(self, delta)
			OnVerticalScroll(self, delta)
		end)

		local SetText = AddTimerFrame.pickSpell.dropdown.list.text.SetText
		AddTimerFrame.pickSpell.dropdown.list.text.SetText = function(self, text)
			SetText(self, text)
		end

		button_OnClick(self)
		self:SetScript("OnClick", button_OnClick)
	end)

	AddTimerFrame.cancelButton = CreateButton(AddTimerFrame, "JSBOptions_AddTimer_CancelButton", "Cancel", 60, 24)
	AddTimerFrame.cancelButton:SetPoint("BOTTOM", AddTimerFrame, "BOTTOM", -34, 6)
	AddTimerFrame.cancelButton:SetScript("OnClick", function(self) self:GetParent():Hide() end)

	AddTimerFrame.saveButton = CreateButton(AddTimerFrame, "JSBOptions_AddTimer_SaveButton", editTimer ~= nil and JSHB.locale.savebutton1 or JSHB.locale.savebutton2, 60, 24)
	AddTimerFrame.saveButton:SetPoint("BOTTOM", AddTimerFrame, "BOTTOM", 34, 6)
	AddTimerFrame.saveButton:SetScript("OnClick", function(self)
		local idx = #AddTimerFrame.specTimers + 1
		local i

		-- Add
		if not AddTimerFrame.editTimer then
			AddTimerFrame.specTimers[idx] = { 0, 0, 0, 0 }
			for i=1,#AddTimerFrame.spellTable do
				if AddTimerFrame.spellTable[i][1] == AddTimerFrame.pickSpell:GetValue() then 
					AddTimerFrame.specTimers[idx][1] = AddTimerFrame.spellTable[i][2]
					break
				end
			end
			AddTimerFrame.specTimers[idx][2] = (UIDropDownMenu_GetSelectedValue(AddTimerFrame.pickDurOrCD.dropdown) == 1) and true or false
			AddTimerFrame.specTimers[idx][3] = (UIDropDownMenu_GetSelectedValue(AddTimerFrame.pickPlayerOrTarget.dropdown) == 1) and true or false
			AddTimerFrame.specTimers[idx][4] = UIDropDownMenu_GetSelectedValue(AddTimerFrame.pickLocation.dropdown)

		-- Update
		else
			for i=1,#AddTimerFrame.spellTable do
				if AddTimerFrame.spellTable[i][1] == AddTimerFrame.pickSpell:GetValue() then 
					AddTimerFrame.specTimers[AddTimerFrame.editTimer][1] = AddTimerFrame.spellTable[i][2]
					break
				end
			end
			AddTimerFrame.specTimers[AddTimerFrame.editTimer][2] = (UIDropDownMenu_GetSelectedValue(AddTimerFrame.pickDurOrCD.dropdown) == 1) and true or false
			AddTimerFrame.specTimers[AddTimerFrame.editTimer][3] = (UIDropDownMenu_GetSelectedValue(AddTimerFrame.pickPlayerOrTarget.dropdown) == 1) and true or false
			AddTimerFrame.specTimers[AddTimerFrame.editTimer][4] = UIDropDownMenu_GetSelectedValue(AddTimerFrame.pickLocation.dropdown)
		end
		table.sort(AddTimerFrame.specTimers, function(a,b) return select(1, GetSpellInfo(a[1]))<select(1, GetSpellInfo(b[1])) end)

		if AddTimerFrame.spec == "bm" then
			RefreshConfigurationSettingsBM()
		elseif AddTimerFrame.spec == "mm" then
			RefreshConfigurationSettingsMM()
		else
			RefreshConfigurationSettingsSV()
		end

		AddTimerFrame:Hide()
	end)
end


RefreshConfigurationSettingsBM = function()

	local x
	for x=1,(#JSHB.options.bm.Timers + 1) do
		if _G["JSBOptions_timers_bm"..x] ~= nil then
			_G["JSBOptions_timers_bm"..x]:Hide()
			_G["JSBOptions_timers_bm"..x] = nil
		end
	end

	if (not JSHB.options.bm.Timers) or (#JSHB.options.bm.Timers == 0) then 
		JSHB.options.bm.text1:SetText(JSHB.locale.nottracking .. JSHB.locale.namebm)
		return 
	else
		JSHB.options.bm.text1:SetText(format(JSHB.locale.currentlytracking1 .. JSHB.locale.namebm .. JSHB.locale.currentlytracking2, #JSHB.options.bm.Timers))
	end

	for x=1,#JSHB.options.bm.Timers do
		createTimerFrame("bm", JSHB.options.bm, "JSBOptions_timers_bm"..x, x, JSHB.options.bm.text1, -((x-1)*37))
	end
end


RefreshConfigurationSettingsMM = function()

	local x
	for x=1,(#JSHB.options.mm.Timers + 1) do
		if _G["JSBOptions_timers_mm"..x] ~= nil then
			_G["JSBOptions_timers_mm"..x]:Hide()
			_G["JSBOptions_timers_mm"..x] = nil
		end
	end

	if (not JSHB.options.mm.Timers) or (#JSHB.options.mm.Timers == 0) then 
		JSHB.options.mm.text1:SetText(JSHB.locale.nottracking .. JSHB.locale.namemm)
		return 
	else
		JSHB.options.mm.text1:SetText(format(JSHB.locale.currentlytracking1 .. JSHB.locale.namemm .. JSHB.locale.currentlytracking2, #JSHB.options.mm.Timers))
	end

	for x=1,#JSHB.options.mm.Timers do
		createTimerFrame("mm", JSHB.options.mm, "JSBOptions_timers_mm"..x, x, JSHB.options.mm.text1, -((x-1)*37))
	end
end


RefreshConfigurationSettingsSV = function()

	local x
	for x=1,(#JSHB.options.sv.Timers + 1) do
		if _G["JSBOptions_timers_sv"..x] ~= nil then
			_G["JSBOptions_timers_sv"..x]:Hide()
			_G["JSBOptions_timers_sv"..x] = nil
		end
	end

	if (not JSHB.options.sv.Timers) or (#JSHB.options.sv.Timers == 0) then 
		JSHB.options.sv.text1:SetText(JSHB.locale.nottracking .. JSHB.locale.namesv)
		return 
	else
		JSHB.options.sv.text1:SetText(format(JSHB.locale.currentlytracking1 .. JSHB.locale.namesv .. JSHB.locale.currentlytracking2, #JSHB.options.sv.Timers))
	end

	for x=1,#JSHB.options.sv.Timers do
		createTimerFrame("sv", JSHB.options.sv, "JSBOptions_timers_sv"..x, x, JSHB.options.sv.text1, -((x-1)*37))
	end
end


local function ApplyDefaultConfigurationSettings()
	local db = JSHB.mainframe.db

	local x, y
	for x,y in pairs(JSHB.defaults["main"]) do 
		db[x] = y
	end

	JSHB.reconfigureAddon()
	JSHB.setupOptionsPane()
end


local function ApplyDefaultConfigurationSettingsGeneral()
	local db = JSHB.mainframe.db

	local x, y
	for x,y in pairs(JSHB.defaults["general"]) do 
		db[x] = y
	end

	JSHB.reconfigureAddon()
	JSHB.setupOptionsPane()
end


local function ApplyDefaultConfigurationSettingsStyleSize()
	local db = JSHB.mainframe.db

	local x, y
	for x,y in pairs(JSHB.defaults["style"]) do 
		db[x] = y
	end

	JSHB.reconfigureAddon()
	JSHB.setupOptionsPane()
end


local function ApplyDefaultConfigurationSettingsFontsTextures()
	local db = JSHB.mainframe.db

	local x, y
	for x,y in pairs(JSHB.defaults["fontstextures"]) do 
		db[x] = y
	end

	JSHB.reconfigureAddon()
	JSHB.setupOptionsPane()
end


local function ApplyDefaultConfigurationSettingsColors()
	local db = JSHB.mainframe.db

	local x, y
	for x,y in pairs(JSHB.defaults["colors"]) do 
		db[x] = y
	end

	JSHB.reconfigureAddon()
	JSHB.setupOptionsPane()
end


local function ApplyDefaultConfigurationSettingsBM()
	local db = JSHB.mainframe.db

	local x
	for x=1,(#JSHB.options.bm.Timers + 1) do
		if _G["JSBOptions_timers_bm"..x] ~= nil then
			_G["JSBOptions_timers_bm"..x]:Hide()
			_G["JSBOptions_timers_bm"..x] = nil
		end
	end

	db.timers["bm"] = JSHB.defaults["bm"]

	table.sort(db.timers["bm"], function(a,b) return select(1, GetSpellInfo(a[1]))<select(1, GetSpellInfo(b[1])) end)
	JSHB.options.bm.Timers = db.timers["bm"]

	JSHB.reconfigureAddon()
	JSHB.setupOptionsPane()
end


local function ApplyDefaultConfigurationSettingsMM()
	local db = JSHB.mainframe.db

	local x
	for x=1,(#JSHB.options.mm.Timers + 1) do
		if _G["JSBOptions_timers_mm"..x] ~= nil then
			_G["JSBOptions_timers_mm"..x]:Hide()
			_G["JSBOptions_timers_mm"..x] = nil
		end
	end

	db.timers["mm"] = JSHB.defaults["mm"]

	table.sort(db.timers["mm"], function(a,b) return select(1, GetSpellInfo(a[1]))<select(1, GetSpellInfo(b[1])) end)
	JSHB.options.mm.Timers = db.timers["mm"]

	JSHB.reconfigureAddon()
	JSHB.setupOptionsPane()
end


local function ApplyDefaultConfigurationSettingsSV()
	local db = JSHB.mainframe.db

	local x
	for x=1,(#JSHB.options.sv.Timers + 1) do
		if _G["JSBOptions_timers_sv"..x] ~= nil then
			_G["JSBOptions_timers_sv"..x]:Hide()
			_G["JSBOptions_timers_sv"..x] = nil
		end
	end

	db.timers["sv"] = JSHB.defaults["sv"]

	table.sort(db.timers["sv"], function(a,b) return select(1, GetSpellInfo(a[1]))<select(1, GetSpellInfo(b[1])) end)
	JSHB.options.sv.Timers = db.timers["sv"]

	JSHB.reconfigureAddon()
	JSHB.setupOptionsPane()
end


CreateButton = function(parent, frameName, label, width, height, onclick, fontObject)
	local button = CreateFrame("Button", frameName, parent, "UIPanelButtonTemplate")
	button:SetWidth(width or 100)
	button:SetHeight(height or 20)
	button:SetText(label)

	if onclick then
		button:SetScript("OnClick", onclick)
	end
	if fontObject then
		button:SetNormalFontObject(fontObject);
		button:SetHighlightFontObject(fontObject);		
	end
	if _G[frameName.."Text"]:GetStringWidth() > button:GetWidth() then
		button:SetWidth( _G[frameName.."Text"]:GetStringWidth() + 25 )
	end

	return button
end


local function CreateCheckButton(ref, parent, label)
	local checkbutton = CreateFrame("CheckButton", ref, parent, "InterfaceOptionsCheckButtonTemplate")
	_G[ref.."Text"]:SetText(label)
	_G[ref.."Text"]:SetJustifyH("LEFT")
	checkbutton.GetValue = function() if checkbutton:GetChecked() then return true else return false end end
	checkbutton.SetValue = checkbutton.SetChecked
	return checkbutton
end


local function onValueChanged(frametext, text, value)
	return function(self, value) frametext:SetFormattedText(text, value) end
end


local function CreateSlider(frametitle, parent, text, low, high, step, framewidth, frameheight)
	local slider = CreateFrame('Slider', frametitle, parent, 'OptionsSliderTemplate')
	slider:SetMinMaxValues(low, high)
	slider:SetValueStep(step)
	slider:SetWidth(framewidth)
	slider:SetHeight(frameheight)
	_G[frametitle..'Text']:SetFormattedText(text, slider:GetValue())
	slider:SetScript("OnValueChanged", onValueChanged(_G[frametitle..'Text'], text, slider:GetValue()))
	return slider
end


local function CreateColorSelector(frametitle, parent, text, textwidth, withalpha)
	--- Color select texture with wheel and value
	local colorselect = CreateFrame("ColorSelect", frametitle, parent)
	colorselect:SetWidth(withalpha and (101) or 64)
	colorselect:SetHeight(64)
	
	-- create a color wheel
	local colorwheel = colorselect:CreateTexture()
	colorwheel:SetWidth(64)
	colorwheel:SetHeight(64)
	colorwheel:SetPoint("TOPLEFT", colorselect, "TOPLEFT", 5, 0)
	colorselect:SetColorWheelTexture(colorwheel)
	
	-- create the colorpicker
	local pickertexture = colorselect:CreateTexture()
	pickertexture:SetTexture("Interface\\Buttons\\UI-ColorPicker-Buttons")
	pickertexture:SetWidth(10)
	pickertexture:SetHeight(10)
	pickertexture:SetTexCoord(0,0.15625, 0, 0.625)
	colorselect:SetColorWheelThumbTexture(pickertexture)
	
	if withalpha then
		-- create the alpha bar
		local colorvalue = colorselect:CreateTexture()
		colorvalue:SetWidth(32)
		colorvalue:SetHeight(64)
		colorvalue:SetPoint("LEFT", colorwheel, "RIGHT", 10, -3)
		colorselect:SetColorValueTexture(colorvalue)
	
		-- create the alpha arrows
		local thumbtexture = colorselect:CreateTexture()
		thumbtexture:SetTexture("Interface\\Buttons\\UI-ColorPicker-Buttons")
		thumbtexture:SetWidth(48)
		thumbtexture:SetHeight(14)
		thumbtexture:SetTexCoord(0.25, 1, 0.875, 0)
		colorselect:SetColorValueThumbTexture(thumbtexture)
	end
	
	-- Add in the text below the picker for example of color
	colorselect.sampletext = colorselect:CreateFontString(frametitle.."SampleText", "ARTWORK")
	colorselect.sampletext:SetFontObject(GameFontNormal)
	colorselect.sampletext:SetText(text)
	colorselect.sampletext:SetJustifyH("CENTER")
	colorselect.sampletext:SetWidth(textwidth)
	colorselect.sampletext:SetPoint("BOTTOM", colorselect, "TOP", 5, 4)

	-- Add script to change the color of the text font to match selected color
	colorselect:SetScript("OnColorSelect", function() colorselect.sampletext:SetTextColor(colorselect:GetColorRGB()) end)

	return colorselect
end


local ii = 0
CreateDropdown = function(parent, name, init)
	ii = ii + 1

	local frame = CreateFrame("Frame", nil, parent)
	frame:SetHeight(42)
	frame:SetWidth(162)
	frame:EnableMouse(true)
	frame:SetScript("OnEnter", Frame_OnEnter)
	frame:SetScript("OnLeave", Frame_OnLeave)
	frame:SetScript("OnHide", OnHide)

	local dropdown = CreateFrame("Frame", "JSHB_ConfigDropdown" .. ii, frame)
	dropdown:SetPoint("TOPLEFT", frame, -16, -14)
	dropdown:SetPoint("TOPRIGHT", frame, 16, -14)
	dropdown:SetHeight(32)

	local ltex = dropdown:CreateTexture(dropdown:GetName() .. "Left", "ARTWORK")
	ltex:SetTexture("Interface\\Glues\\CharacterCreate\\CharacterCreate-LabelFrame")
	ltex:SetTexCoord(0, 0.1953125, 0, 1)
	ltex:SetPoint("TOPLEFT", dropdown, 0, 17)
	ltex:SetWidth(25)
	ltex:SetHeight(64)

	local rtex = dropdown:CreateTexture(nil, "BORDER")
	rtex:SetTexture("Interface\\Glues\\CharacterCreate\\CharacterCreate-LabelFrame")
	rtex:SetTexCoord(0.8046875, 1, 0, 1)
	rtex:SetPoint("TOPRIGHT", dropdown, 0, 17)
	rtex:SetWidth(25)
	rtex:SetHeight(64)

	local mtex = dropdown:CreateTexture(nil, "BORDER")
	mtex:SetTexture("Interface\\Glues\\CharacterCreate\\CharacterCreate-LabelFrame")
	mtex:SetTexCoord(0.1953125, 0.8046875, 0, 1)
	mtex:SetPoint("LEFT", ltex, "RIGHT")
	mtex:SetPoint("RIGHT", rtex, "LEFT")
	mtex:SetHeight(64)

	local label = frame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	label:SetPoint("BOTTOMLEFT", dropdown, "TOPLEFT", 20, 0)
	label:SetPoint("BOTTOMRIGHT", dropdown, "TOPRIGHT", -20, 0)
	label:SetJustifyH("LEFT")
	label:SetText(name)

	local value = dropdown:CreateFontString(dropdown:GetName() .. "Text", "OVERLAY", "GameFontHighlightSmall")
	value:SetPoint("LEFT", ltex, 26, 2)
	value:SetPoint("RIGHT", rtex, -43, 2)
	value:SetJustifyH("LEFT")
	value:SetHeight(10)

	local button = CreateFrame("Button", nil, dropdown)
	button:SetPoint("TOPRIGHT", rtex, -16, -18)
	button:SetWidth(24)
	button:SetHeight(24)
	button:SetScript("OnEnter", Button_OnEnter)
	button:SetScript("OnLeave", Frame_OnLeave)
	button:SetScript("OnClick", OnClick)

	button:SetNormalTexture("Interface\\ChatFrame\\UI-ChatIcon-ScrollDown-Up")
	button:SetPushedTexture("Interface\\ChatFrame\\UI-ChatIcon-ScrollDown-Down")
	button:SetDisabledTexture("Interface\\ChatFrame\\UI-ChatIcon-ScrollDown-Disabled")
	button:SetHighlightTexture("Interface\\Buttons\\UI-Common-MouseHilight")
	button:GetHighlightTexture():SetBlendMode("ADD")

	frame.button = button
	frame.dropdown = dropdown
	frame.labelText = label
	frame.valueText = value

	frame.GetValue = GetValue
	frame.SetValue = SetValue

	if init then
		UIDropDownMenu_Initialize(dropdown, init)
	end

	return frame
end


local function ListButton_OnClick(self)
	local dropdown = self:GetParent():GetParent()
	dropdown.selected = self.value
	dropdown.list:Hide()

	local container = dropdown:GetParent()
	container.valueText:SetText(self.value)

	if container.OnValueChanged then
		container:OnValueChanged(self.value)
	end

	PlaySound("UChatScrollButton")
end


local function CreateListButton(parent)
	local button = CreateFrame("Button", nil, parent)
	button:SetHeight(UIDROPDOWNMENU_BUTTON_HEIGHT)

	button.label = button:CreateFontString()
	button.label:SetFont((GameFontHighlightSmallLeft:GetFont()), UIDROPDOWNMENU_DEFAULT_TEXT_HEIGHT)
	button.label:SetJustifyH("LEFT")
	button.label:SetPoint("LEFT", 27, 0)

	button.check = button:CreateTexture(nil, "ARTWORK")
	button.check:SetWidth(24)
	button.check:SetHeight(24)
	button.check:SetTexture("Interface\\Buttons\\UI-CheckBox-Check")
	button.check:SetPoint("LEFT")

	local highlight = button:CreateTexture(nil, "BACKGROUND")
	highlight:SetAllPoints(button)
	highlight:SetTexture("Interface\\QuestFrame\\UI-QuestTitleHighlight")
	highlight:SetAlpha(0.6)
	highlight:SetBlendMode("ADD")
	highlight:Hide()
	button:SetHighlightTexture(highlight)

	button:SetScript("OnClick", ListButton_OnClick)

	return button
end


local function UpdateList(self)
	local buttons = self.buttons
	local items = self:GetParent().items
	local listSize = min(#items, 10) -- 10 = Max number of items from the list to display at one time

	local scrollFrame = self.scrollFrame
	local offset = scrollFrame.offset
	FauxScrollFrame_Update(scrollFrame, #items, listSize, UIDROPDOWNMENU_BUTTON_HEIGHT)

	local selected = self:GetParent().selected

	for i = 1, listSize do
		local index = i + offset
		local button = self.buttons[i]

		local item = items[index]
		if item then
			button.value = item
			button.label:SetText(item)

			if item == selected then
				button.check:Show()
			else
				button.check:Hide()
			end

			button:SetWidth(self.width)
			button:Show()
		else
			button.value = nil
			button.label:SetText()
			button.check:Hide()
			button:Hide()
		end
	end

	for i = listSize + 1, #buttons do
		buttons[i]:Hide()
	end

	if self.scrollFrame:IsShown() then
		self:SetWidth(self.width + 50)
	else
		self:SetWidth(self.width + 30)
	end
	self:SetHeight((listSize * UIDROPDOWNMENU_BUTTON_HEIGHT) + (UIDROPDOWNMENU_BORDER_HEIGHT * 2))
end


local function CreateList(parent)
	local list = CreateFrame("Button", parent:GetName() .. "List", parent)
	list:SetToplevel(true)
	list:Raise()

	list.text = list:CreateFontString()
	list.text:SetFont("Fonts\\FRIZQT__.ttf", UIDROPDOWNMENU_DEFAULT_TEXT_HEIGHT + 2)

	list.buttons = setmetatable({}, { __index = function(t, i)
		local button = CreateListButton(list)
		if i > 1 then
			button:SetPoint("TOPLEFT", t[i-1], "BOTTOMLEFT")
		else
			button:SetPoint("TOPLEFT", 15, -15)
		end
		t[i] = button

		return button
	end })

	list.scrollFrame = CreateFrame("ScrollFrame", list:GetName() .. "ScrollFrame", list, "FauxScrollFrameTemplate")
	list.scrollFrame:SetPoint("TOPLEFT", 6, -7)-- 12, -14
	list.scrollFrame:SetPoint("BOTTOMRIGHT", -30, 6)-- -36,13
	list.scrollFrame:SetScript("OnVerticalScroll", function(self, delta)
		FauxScrollFrame_OnVerticalScroll(self, delta, UIDROPDOWNMENU_BUTTON_HEIGHT, function() UpdateList(list) end)
	end)

	list.border = CreateFrame("Frame", nil, list)
	list.border:SetBackdrop({
	  bgFile = "",
	  edgeFile = JSHB.tex[JSHB.tex.getTextureIndex("Blank")][2], 
	  tile = false, tileSize = 0, edgeSize = 1, 
	  insets = { left = -1, right = -1, top = -1, bottom = -1}
	})
	list.border:SetBackdropColor(.1,.1,.1,1)
	list.border:SetBackdropBorderColor(.6,.6,.6,1)
	list.border:SetPoint("TOPLEFT", -1, 1)
	list.border:SetPoint("BOTTOMRIGHT", 1, -1)

	list:SetScript("OnShow", function(self)
		self.width = 0
		for i, item in pairs(self:GetParent().items) do
			self.text:SetText(item)
			self.width = max(self.text:GetWidth() + 60, self.width)
		end
		UpdateList(self)
	end)
	list:SetScript("OnHide", list.Hide)
	list:SetScript("OnClick", list.Hide)
	list:SetPoint("TOPLEFT", parent, "BOTTOMLEFT", 12, 6)
	list:Hide()

	local bgtex = list:CreateTexture(nil, "BACKGROUND", list)
	bgtex:SetAllPoints(list)
	bgtex:SetTexture(0, 0, 0, 1)

	return list
end


local function DropdownButton_OnClick(self)
	local list = self:GetParent().list
	if list then
		if list:IsShown() then 
			list:Hide()
		else
			list:SetToplevel(true)
			list:Raise()
			list:Show()
		end
	else
		local dropdown = self:GetParent()
		dropdown.list = CreateList(dropdown)
		dropdown.list:Show()
	end
end


local function GetValue(self)
	return self.dropdown.selected or self.valueText:GetText()
end


local function SetValue(self, value, text)
	self.valueText:SetText(text or value)
	self.dropdown.selected = value
end


local function SetValue(self, value, text)
	self.valueText:SetText(text or value)
	self.dropdown.selected = value
end


function CreateScrollingDropdown(parent, name, items)
	local container = CreateDropdown(parent, name)
	local singleColTable = { }
	local i

	for i=1,#items do
		singleColTable[i] = items[i][1]
	end

	if type(singleColTable) == "table" then
		container.button:SetScript("OnClick", DropdownButton_OnClick)
		container.dropdown.items = singleColTable
	end

	container.SetValue = SetValue

	return container
end


JSHB.options = CreateFrame( "Frame", "JSHB.options", UIParent )

-- Register in the Interface Addon Options GUI
JSHB.options.name = "JS' Hunter Bar"
JSHB.options.okay = ApplyConfigurationSettings
JSHB.options.cancel = CancelConfigurationSettings
JSHB.options.default = ApplyDefaultConfigurationSettings

JSHB.options.enablebarlock = 
	CreateCheckButton("JSBOptions_enablebarlock", JSHB.options, JSHB.locale.enablebarlock)
JSHB.options.enablebarlock:SetPoint("TOPLEFT", 10, -10)

InterfaceOptions_AddCategory(JSHB.options)


 -- Create general options panel
JSHB.options.general = CreateFrame( "Frame", "JSHB.optionsGeneral", JSHB.options)
JSHB.options.general.name = JSHB.locale.namegeneral
JSHB.options.general.parent = JSHB.options.name
JSHB.options.general.default = ApplyDefaultConfigurationSettingsGeneral

local optgen = JSHB.options.general

optgen.enableautoshotbar = 
	CreateCheckButton("JSBOptions_enableautoshotbar", optgen, JSHB.locale.enableautoshotbar)
optgen.enableautoshotbar:SetPoint("TOPLEFT", 10, -10)

optgen.enableautoshottext = 
	CreateCheckButton("JSBOptions_enableautoshottext", optgen, JSHB.locale.enableautoshottext)
optgen.enableautoshottext:SetPoint("TOPLEFT", optgen.enableautoshotbar, "TOPLEFT", 10, -25)

optgen.enablemaintick = 
	CreateCheckButton("JSBOptions_enablemaintick", optgen, JSHB.locale.enablemaintick)
optgen.enablemaintick:SetPoint("TOPLEFT", optgen.enableautoshottext, "TOPLEFT", -10, -25)

optgen.enablehuntersmarkwarning = 
	CreateCheckButton("JSBOptions_enablehuntersmarkwarning", optgen, JSHB.locale.enablehuntersmarkwarning)
optgen.enablehuntersmarkwarning:SetPoint("TOPLEFT", optgen.enablemaintick, "TOPLEFT", 0, -25)

optgen.enabletranqannounce = 
	CreateCheckButton("JSBOptions_enabletranqannounce", optgen, JSHB.locale.enabletranqannounce)
optgen.enabletranqannounce:SetPoint("TOPLEFT", optgen.enablehuntersmarkwarning, "TOPLEFT", 0, -25)

optgen.tranqannouncechannel = CreateDropdown(optgen, JSHB.locale.tranqannouncechannel, function()
	local i
	for i=1,#JSHB.tranqannouncechannel do
		info.text = JSHB.tranqannouncechannel[i][1]
		info.value = JSHB.tranqannouncechannel[i][2] or 1
		info.func = function(self) optgen.tranqannouncechannel:SetValue(self.value, self.text) end
		info.checked = false
		UIDropDownMenu_AddButton(info)
	end
end)
optgen.tranqannouncechannel:SetPoint("TOPLEFT", optgen.enabletranqannounce, "TOPLEFT", 25, -25)
optgen.tranqannouncechannel:SetPoint("TOPRIGHT", optgen.enabletranqannounce, "TOPLEFT", 160, -45)

optgen.enabletranqalert = 
	CreateCheckButton("JSBOptions_enabletranqalert", optgen, JSHB.locale.enabletranqalert)
optgen.enabletranqalert:SetPoint("TOPLEFT", optgen.tranqannouncechannel, "TOPLEFT", -25, -50)

optgen.enablecctimers = 
	CreateCheckButton("JSBOptions_enablecctimers", optgen, JSHB.locale.enablecctimers)
optgen.enablecctimers:SetPoint("TOPLEFT", optgen.enabletranqalert, "TOPLEFT", 0, -25)

optgen.enableprediction = 
	CreateCheckButton("JSBOptions_enableprediction", optgen, JSHB.locale.enableprediction)
optgen.enableprediction:SetPoint("TOPLEFT", optgen.enablecctimers, "TOPLEFT", 0, -25)

optgen.enabletimers = 
	CreateCheckButton("JSBOptions_enabletimers", optgen, JSHB.locale.enabletimers)
optgen.enabletimers:SetPoint("TOPLEFT", optgen.enableprediction, "TOPLEFT", 0, -25)

optgen.timerfontposition = CreateDropdown(optgen, JSHB.locale.timerfontposition, function()
	local i
	for i=1,#JSHB.timerfontpositions do
		info.text = JSHB.timerfontpositions[i][1]
		info.value = JSHB.timerfontpositions[i][2]
		info.func = function(self) optgen.timerfontposition:SetValue(self.value, self.text) end
		info.checked = false
		UIDropDownMenu_AddButton(info)
	end
end)
optgen.timerfontposition:SetPoint("TOPLEFT", optgen.enabletimers, "TOPLEFT", 25, -25)
optgen.timerfontposition:SetPoint("TOPRIGHT", optgen.enabletimers, "TOPLEFT", 160, -45)

optgen.enabletimerstext = 
	CreateCheckButton("JSBOptions_enabletimerstext", optgen, JSHB.locale.enabletimerstext)
optgen.enabletimerstext:SetPoint("TOPLEFT", optgen.timerfontposition, "TOPLEFT", -25, -50)

optgen.enabletimertenths = 
	CreateCheckButton("JSBOptions_enabletimertenths", optgen, JSHB.locale.enabletimertenths)
optgen.enabletimertenths:SetPoint("TOPLEFT", optgen.enabletimerstext, "TOPLEFT", 10, -25)

optgen.enabledebuffalert = 
	CreateCheckButton("JSBOptions_enabledebuffalert", optgen, JSHB.locale.enabledebuffalert)
optgen.enabledebuffalert:SetPoint("TOPLEFT", optgen.enabletimertenths, "TOPLEFT", -10, -25)

InterfaceOptions_AddCategory(optgen)


-- Create Style & Size options panel
JSHB.options.stylesizescrollframeparent = CreateFrame("Frame", "JSHB.optionsStyleSizeScrollFrameParent", JSHB.options)
JSHB.options.stylesizescrollframeparent.name = JSHB.locale.namestylesize
JSHB.options.stylesizescrollframeparent.parent = JSHB.options.name
JSHB.options.stylesizescrollframeparent.default = ApplyDefaultConfigurationSettingsStyleSize

JSHB.options.stylesizescrollframe = CreateFrame("ScrollFrame", "JSHB.optionsStyleSizeScrollFrame",
	JSHB.options.stylesizescrollframeparent, "UIPanelScrollFrameTemplate")
JSHB.options.stylesizescrollframe:SetPoint("TOPLEFT", JSHB.options.stylesizescrollframeparent, "TOPLEFT", 0, -8)
JSHB.options.stylesizescrollframe:SetPoint("BOTTOMLEFT", JSHB.options.stylesizescrollframeparent, "BOTTOMLEFT", 0, 8)
JSHB.options.stylesizescrollframe:SetPoint("RIGHT", JSHB.options.stylesizescrollframeparent, "RIGHT", -28, 0)
JSHB.options.stylesizescrollframe:SetFrameLevel(JSHB.options.stylesizescrollframeparent:GetFrameLevel())

JSHB.options.stylesize = CreateFrame("Frame", "JSHB.optionsStyleSize", JSHB.options.stylesizescrollframe)
JSHB.options.stylesize:SetWidth(350)
JSHB.options.stylesize:SetHeight(450)

local optcs = JSHB.options.stylesize

optcs.classcolored = 
	CreateCheckButton("JSBOptions_classcolored", optcs, JSHB.locale.classcolored)
optcs.classcolored:SetPoint("TOPLEFT", 10, -12)

optcs.classcoloredprediction = 
	CreateCheckButton("JSBOptions_classcoloredprediction", optcs, JSHB.locale.classcoloredprediction)
optcs.classcoloredprediction:SetPoint("TOPLEFT", optcs.classcolored, "TOPLEFT", 0, -32)

optcs.enabletukui = 
	CreateCheckButton("JSBOptions_enabletukui", optcs, JSHB.locale.enabletukui)
optcs.enabletukui:SetPoint("TOPLEFT", optcs.classcoloredprediction, "TOPLEFT", 0, -29)

optcs.enabletukuitimers = 
	CreateCheckButton("JSBOptions_enabletukuitimers", optcs, JSHB.locale.enabletukuitimers)
optcs.enabletukuitimers:SetPoint("TOPLEFT", optcs.enabletukui, "TOPLEFT", 10, -30)

optcs.enablehighcolorwarning = 
	CreateCheckButton("JSBOptions_enablehighcolorwarning", optcs, JSHB.locale.enablehighcolorwarning)
optcs.enablehighcolorwarning:SetPoint("TOPLEFT", optcs.enabletukuitimers, "TOPLEFT", -10, -30)

optcs.focushighthreshold =
	CreateSlider("JSBOptions_focushighthreshold", optcs, JSHB.locale.focushighthreshold, 50, 90, 1, 340, 15)
optcs.focushighthreshold:SetPoint("TOPLEFT", optcs.enablehighcolorwarning, "TOPLEFT", 10, -55)

optcs.barwidth =
	CreateSlider("JSBOptions_barwidth", optcs, JSHB.locale.barwidth, 50, 400, 1, 340, 15)
optcs.barwidth:SetPoint("TOPLEFT", optcs.focushighthreshold, "TOPLEFT", 0, -45)

optcs.barheight =
	CreateSlider("JSBOptions_barheight", optcs, JSHB.locale.barheight, 8, 200, 1, 340, 15)
optcs.barheight:SetPoint("TOPLEFT", optcs.barwidth, "TOPLEFT", 0, -45)

optcs.iconsize =
	CreateSlider("JSBOptions_iconsize", optcs, JSHB.locale.iconsize, 0, 50, 1, 340, 15)
optcs.iconsize:SetPoint("TOPLEFT", optcs.barheight, "TOPLEFT", 0, -55)

optcs.cciconsize =
	CreateSlider("JSBOptions_cciconsize", optcs, JSHB.locale.cciconsize, 8, 50, 1, 340, 15)
optcs.cciconsize:SetPoint("TOPLEFT", optcs.iconsize, "TOPLEFT", 0, -45)

optcs.markiconsize =
	CreateSlider("JSBOptions_markiconsize", optcs, JSHB.locale.markiconsize, 8, 50, 1, 340, 15)
optcs.markiconsize:SetPoint("TOPLEFT", optcs.cciconsize, "TOPLEFT", 0, -45)

optcs.taiconsize =
	CreateSlider("JSBOptions_taiconsize", optcs, JSHB.locale.taiconsize, 8, 50, 1, 340, 15)
optcs.taiconsize:SetPoint("TOPLEFT", optcs.markiconsize, "TOPLEFT", 0, -45)

optcs.debufficonsize =
	CreateSlider("JSBOptions_debufficonsize", optcs, JSHB.locale.debufficonsize, 8, 60, 1, 340, 15)
optcs.debufficonsize:SetPoint("TOPLEFT", optcs.taiconsize, "TOPLEFT", 0, -45)

optcs.alphabackdrop =
	CreateSlider("JSBOptions_alphabackdrop", optcs, JSHB.locale.alphabackdrop, 0, 100, 5, 340, 15)
optcs.alphabackdrop:SetPoint("TOPLEFT", optcs.debufficonsize, "TOPLEFT", 0, -55)

optcs.alphazeroooc =
	CreateSlider("JSBOptions_alphazeroooc", optcs, JSHB.locale.alphazeroooc, 0, 100, 5, 340, 15)
optcs.alphazeroooc:SetPoint("TOPLEFT", optcs.alphabackdrop, "TOPLEFT", 0, -55)

optcs.alphamaxooc =
	CreateSlider("JSBOptions_alphamaxooc", optcs, JSHB.locale.alphamaxooc, 0, 100, 5, 340, 15)
optcs.alphamaxooc:SetPoint("TOPLEFT", optcs.alphazeroooc, "TOPLEFT", 0, -55)

optcs.alphanormooc =
	CreateSlider("JSBOptions_alphanormooc", optcs, JSHB.locale.alphanormooc, 0, 100, 5, 340, 15)
optcs.alphanormooc:SetPoint("TOPLEFT", optcs.alphamaxooc, "TOPLEFT", 0, -55)

optcs.alphazero =
	CreateSlider("JSBOptions_alphazero", optcs, JSHB.locale.alphazero, 0, 100, 5, 340, 15)
optcs.alphazero:SetPoint("TOPLEFT", optcs.alphanormooc, "TOPLEFT", 0, -55)

optcs.alphamax =
	CreateSlider("JSBOptions_alphamax", optcs, JSHB.locale.alphamax, 0, 100, 5, 340, 15)
optcs.alphamax:SetPoint("TOPLEFT", optcs.alphazero, "TOPLEFT", 0, -55)

optcs.alphanorm =
	CreateSlider("JSBOptions_alphanorm", optcs, JSHB.locale.alphanorm, 0, 100, 5, 340, 15)
optcs.alphanorm:SetPoint("TOPLEFT", optcs.alphamax, "TOPLEFT", 0, -55)

JSHB.options.stylesizescrollframe:SetScrollChild(optcs)
InterfaceOptions_AddCategory(JSHB.options.stylesizescrollframeparent)


-- Create Fonts & Textures options panel
JSHB.options.fontstextures = CreateFrame( "Frame", "JSHB.optionsFontsTextures", JSHB.options)
JSHB.options.fontstextures.name = JSHB.locale.namefontstextures
JSHB.options.fontstextures.parent = JSHB.options.name
JSHB.options.fontstextures.default = ApplyDefaultConfigurationSettingsFontsTextures

local optft = JSHB.options.fontstextures

optft.barfont = CreateScrollingDropdown(optft, JSHB.locale.barfont, JSHB.fnts)
optft.barfont:SetPoint("TOPLEFT", 15, -12)
optft.barfont:SetValue("Arial")
local button_OnClick = optft.barfont.button:GetScript("OnClick")
optft.barfont.button:SetScript("OnClick", function(self)
	button_OnClick(self)
	optft.barfont.dropdown.list:Hide()

	local function SetButtonFonts(self)
		local buttons = optft.barfont.dropdown.list.buttons
		for i = 1, #buttons do
			local button = buttons[i]
			if button.value and button:IsShown() then
				button.label:SetFont(JSHB.fnts[JSHB.fnts.getFontIndex(button.value)][2],
					UIDROPDOWNMENU_DEFAULT_TEXT_HEIGHT)
			end
		end
	end

	local OnShow = optft.barfont.dropdown.list:GetScript("OnShow")
	optft.barfont.dropdown.list:SetScript("OnShow", function(self)
		OnShow(self)
		SetButtonFonts(self)
	end)

	local OnVerticalScroll = optft.barfont.dropdown.list.scrollFrame:GetScript("OnVerticalScroll")
	optft.barfont.dropdown.list.scrollFrame:SetScript("OnVerticalScroll", function(self, delta)
		OnVerticalScroll(self, delta)
		SetButtonFonts(self)
	end)

	local SetText = optft.barfont.dropdown.list.text.SetText
	optft.barfont.dropdown.list.text.SetText = function(self, text)
		self:SetFont(JSHB.fnts[JSHB.fnts.getFontIndex(text)][2], UIDROPDOWNMENU_DEFAULT_TEXT_HEIGHT + 1)
		SetText(self, text)
	end

	button_OnClick(self)
	self:SetScript("OnClick", button_OnClick)
end)

optft.timerfont = CreateScrollingDropdown(optft, JSHB.locale.timerfont, JSHB.fnts)
optft.timerfont:SetPoint("TOPLEFT", optft.barfont, "TOPLEFT", 0, -55)
optft.timerfont:SetValue("Arial")
local button_OnClick = optft.timerfont.button:GetScript("OnClick")
optft.timerfont.button:SetScript("OnClick", function(self)
	button_OnClick(self)
	optft.timerfont.dropdown.list:Hide()

	local function SetButtonFonts(self)
		local buttons = optft.timerfont.dropdown.list.buttons
		for i = 1, #buttons do
			local button = buttons[i]
			if button.value and button:IsShown() then
				button.label:SetFont(JSHB.fnts[JSHB.fnts.getFontIndex(button.value)][2], UIDROPDOWNMENU_DEFAULT_TEXT_HEIGHT)
			end
		end
	end

	local OnShow = optft.timerfont.dropdown.list:GetScript("OnShow")
	optft.timerfont.dropdown.list:SetScript("OnShow", function(self)
		OnShow(self)
		SetButtonFonts(self)
	end)

	local OnVerticalScroll = optft.timerfont.dropdown.list.scrollFrame:GetScript("OnVerticalScroll")
	optft.timerfont.dropdown.list.scrollFrame:SetScript("OnVerticalScroll", function(self, delta)
		OnVerticalScroll(self, delta)
		SetButtonFonts(self)
	end)

	local SetText = optft.timerfont.dropdown.list.text.SetText
	optft.timerfont.dropdown.list.text.SetText = function(self, text)
		self:SetFont(JSHB.fnts[JSHB.fnts.getFontIndex(text)][2], UIDROPDOWNMENU_DEFAULT_TEXT_HEIGHT + 1)
		SetText(self, text)
	end

	button_OnClick(self)
	self:SetScript("OnClick", button_OnClick)
end)

optft.bartexture = CreateScrollingDropdown(optft, JSHB.locale.bartexture, JSHB.tex)
optft.bartexture:SetPoint("TOPLEFT", optft.timerfont, "TOPLEFT", 0, -55)
optft.bartexture:SetValue("Blizzard")
local button_OnClick = optft.bartexture.button:GetScript("OnClick")
optft.bartexture.button:SetScript("OnClick", function(self)
	button_OnClick(self)
	optft.bartexture.dropdown.list:Hide()

	local function SetButtonTextures(self)
		local buttons = optft.bartexture.dropdown.list.buttons
		for i = 1, #buttons do
			local button = buttons[i]
			if button.value and button:IsShown() then
				local normal = button:CreateTexture(nil, "BACKGROUND")
				normal:SetAllPoints(button)
				normal:SetTexture(JSHB.tex[i][2])
				normal:SetAlpha(0.4)
				normal:SetBlendMode("ADD")
				normal:Hide()
				button:SetNormalTexture(normal)
			end
		end
	end

	local OnShow = optft.bartexture.dropdown.list:GetScript("OnShow")
	optft.bartexture.dropdown.list:SetScript("OnShow", function(self)
		OnShow(self)
		SetButtonTextures(self)
	end)

	local OnVerticalScroll = optft.bartexture.dropdown.list.scrollFrame:GetScript("OnVerticalScroll")
	optft.bartexture.dropdown.list.scrollFrame:SetScript("OnVerticalScroll", function(self, delta)
		OnVerticalScroll(self, delta)
		SetButtonTextures(self)
	end)

	local SetText = optft.bartexture.dropdown.list.text.SetText
	optft.bartexture.dropdown.list.text.SetText = function(self, text) SetText(self, text) end

	button_OnClick(self)
	self:SetScript("OnClick", button_OnClick)
end)

optft.fontoutlined = 
	CreateCheckButton("JSBOptions_fontoutlined", optft, JSHB.locale.fontoutlined)
optft.fontoutlined:SetPoint("TOPLEFT", optft.bartexture, "TOPLEFT", 0, -55)

optft.fontsize =
	CreateSlider("JSBOptions_fontsize", optft, JSHB.locale.fontsize, 8, 30, 1, 340, 15)
optft.fontsize:SetPoint("TOPLEFT", optft.fontoutlined, "TOPLEFT", 5, -45)

optft.fontsizetimers =
	CreateSlider("JSBOptions_fontsizetimers", optft, JSHB.locale.fontsizetimers, 8, 30, 1, 340, 15)
optft.fontsizetimers:SetPoint("TOPLEFT", optft.fontsize, "TOPLEFT", 0, -45)

InterfaceOptions_AddCategory(optft)


-- Create Colors options panel
JSHB.options.colors = CreateFrame( "Frame", "JSHB.optionsColors", JSHB.options)
local optc = JSHB.options.colors
optc.name = JSHB.locale.namecolors
optc.parent = JSHB.options.name
optc.default = ApplyDefaultConfigurationSettingsColors

optc.barcolor =
	CreateColorSelector("JSBOptions_barcolor", optc, JSHB.locale.barcolor, 80)
optc.barcolor:SetPoint("TOPLEFT", 40, -50)

optc.barcolorwarninglow =
	CreateColorSelector("JSBOptions_barcolorwarninglow", optc, JSHB.locale.barcolorwarninglow, 80)
optc.barcolorwarninglow:SetPoint("LEFT", optc.barcolor, "RIGHT", 50, 0)

optc.barcolorwarninghigh =
	CreateColorSelector("JSBOptions_barcolorwarninghigh", optc, JSHB.locale.barcolorwarninghigh, 80)
optc.barcolorwarninghigh:SetPoint("LEFT", optc.barcolorwarninglow, "RIGHT", 50, 0)

optc.autoshotbarcolor =
	CreateColorSelector("JSBOptions_autoshotbarcolor", optc, JSHB.locale.autoshotbarcolor, 80)
optc.autoshotbarcolor:SetPoint("TOPLEFT", optc.barcolor, "TOPLEFT", 0, -120)

optc.predictionbarcolor =
	CreateColorSelector("JSBOptions_barcolorwarninglow", optc, JSHB.locale.predictionbarcolor, 80)
optc.predictionbarcolor:SetPoint("LEFT", optc.autoshotbarcolor, "RIGHT", 50, 0)

optc.predictionbarcolorwarninghigh =
	CreateColorSelector("JSBOptions_barcolorwarninghigh", optc, JSHB.locale.predictionbarcolorwarninghigh, 80)
optc.predictionbarcolorwarninghigh:SetPoint("LEFT", optc.predictionbarcolor, "RIGHT", 50, 0)

InterfaceOptions_AddCategory(optc)


-- Create BM options panel
JSHB.options.bmscrollframeparent = CreateFrame("Frame", "JSHB.optionsBMScrollFrameParent", JSHB.options)
JSHB.options.bmscrollframeparent.name = JSHB.locale.namebm
JSHB.options.bmscrollframeparent.parent = JSHB.options.name
JSHB.options.bmscrollframeparent.default = ApplyDefaultConfigurationSettingsBM
JSHB.options.bmscrollframeparent.refresh = RefreshConfigurationSettingsBM

JSHB.options.bmscrollframe = CreateFrame("ScrollFrame", "JSHB.optionsBMScrollFrame", JSHB.options.bmscrollframeparent, "UIPanelScrollFrameTemplate")
JSHB.options.bmscrollframe:SetPoint("TOPLEFT", JSHB.options.bmscrollframeparent, "TOPLEFT", 0, -8)
JSHB.options.bmscrollframe:SetPoint("BOTTOMLEFT", JSHB.options.bmscrollframeparent, "BOTTOMLEFT", 0, 8)
JSHB.options.bmscrollframe:SetPoint("RIGHT", JSHB.options.bmscrollframeparent, "RIGHT", -28, 0)
JSHB.options.bmscrollframe:SetFrameLevel(JSHB.options.bmscrollframeparent:GetFrameLevel())

JSHB.options.bm = CreateFrame("Frame", "JSHB.optionsBM", JSHB.options.bmscrollframe)
JSHB.options.bm:SetWidth(350)
JSHB.options.bm:SetHeight(450)

local optbm = JSHB.options.bm

optbm.button1 = CreateButton(optbm, "JSBOptions_BMButton1", JSHB.locale.buttonaddtimer, 60, 24)
optbm.button1:SetPoint("TOPLEFT", 14, -10)
optbm.button1:SetScript("OnClick", function() AddTimer("bm", JSHB.options.bm) end)

optbm.text1 = optbm:CreateFontString("JSBOptions_BMText1", "ARTWORK", optbm)
optbm.text1:SetFontObject(GameFontNormal)	
optbm.text1:SetJustifyH("LEFT")
optbm.text1:SetWidth(360)
optbm.text1:SetPoint("TOPLEFT", optbm.button1, "TOPLEFT", 0, -35)

JSHB.options.bmscrollframe:SetScrollChild(optbm)
InterfaceOptions_AddCategory(JSHB.options.bmscrollframeparent)


-- Create MM options panel
JSHB.options.mmscrollframeparent = CreateFrame("Frame", "JSHB.optionsMMScrollFrameParent", JSHB.options)
JSHB.options.mmscrollframeparent.name = JSHB.locale.namemm
JSHB.options.mmscrollframeparent.parent = JSHB.options.name
JSHB.options.mmscrollframeparent.default = ApplyDefaultConfigurationSettingsMM
JSHB.options.mmscrollframeparent.refresh = RefreshConfigurationSettingsMM

JSHB.options.mmscrollframe = CreateFrame("ScrollFrame", "JSHB.optionsMMScrollFrame", JSHB.options.mmscrollframeparent, "UIPanelScrollFrameTemplate")
JSHB.options.mmscrollframe:SetPoint("TOPLEFT", JSHB.options.mmscrollframeparent, "TOPLEFT", 0, -8)
JSHB.options.mmscrollframe:SetPoint("BOTTOMLEFT", JSHB.options.mmscrollframeparent, "BOTTOMLEFT", 0, 8)
JSHB.options.mmscrollframe:SetPoint("RIGHT", JSHB.options.mmscrollframeparent, "RIGHT", -28, 0)
JSHB.options.mmscrollframe:SetFrameLevel(JSHB.options.mmscrollframeparent:GetFrameLevel())

JSHB.options.mm = CreateFrame("Frame", "JSHB.optionsMM", JSHB.options.mmscrollframe)
JSHB.options.mm:SetWidth(350)
JSHB.options.mm:SetHeight(450)

local optmm = JSHB.options.mm

optmm.button1 = CreateButton(optmm, "JSBOptions_MMButton1", JSHB.locale.buttonaddtimer, 60, 24)
optmm.button1:SetPoint("TOPLEFT", 14, -10)
optmm.button1:SetScript("OnClick", function() AddTimer("mm", JSHB.options.mm) end)

optmm.text1 = optmm:CreateFontString("JSBOptions_MMText1", "ARTWORK", optmm)
optmm.text1:SetFontObject(GameFontNormal)	
optmm.text1:SetJustifyH("LEFT")
optmm.text1:SetWidth(360)
optmm.text1:SetPoint("TOPLEFT", optmm.button1, "TOPLEFT", 0, -35)

JSHB.options.mmscrollframe:SetScrollChild(optmm)
InterfaceOptions_AddCategory(JSHB.options.mmscrollframeparent)


-- Create SV options panel
JSHB.options.svscrollframeparent = CreateFrame("Frame", "JSHB.optionsSVScrollFrameParent", JSHB.options)
JSHB.options.svscrollframeparent.name = JSHB.locale.namesv
JSHB.options.svscrollframeparent.parent = JSHB.options.name
JSHB.options.svscrollframeparent.default = ApplyDefaultConfigurationSettingsSV
JSHB.options.svscrollframeparent.refresh = RefreshConfigurationSettingsSV

JSHB.options.svscrollframe = CreateFrame("ScrollFrame", "JSHB.optionsSVScrollFrame", JSHB.options.svscrollframeparent, "UIPanelScrollFrameTemplate")
JSHB.options.svscrollframe:SetPoint("TOPLEFT", JSHB.options.svscrollframeparent, "TOPLEFT", 0, -8)
JSHB.options.svscrollframe:SetPoint("BOTTOMLEFT", JSHB.options.svscrollframeparent, "BOTTOMLEFT", 0, 8)
JSHB.options.svscrollframe:SetPoint("RIGHT", JSHB.options.svscrollframeparent, "RIGHT", -28, 0)
JSHB.options.svscrollframe:SetFrameLevel(JSHB.options.svscrollframeparent:GetFrameLevel())

JSHB.options.sv = CreateFrame("Frame", "JSHB.optionsSV", JSHB.options.svscrollframe)
JSHB.options.sv:SetWidth(350)
JSHB.options.sv:SetHeight(450)

local optsv = JSHB.options.sv

optsv.button1 = CreateButton(optsv, "JSBOptions_SVButton1", JSHB.locale.buttonaddtimer, 60, 24)
optsv.button1:SetPoint("TOPLEFT", 14, -10)
optsv.button1:SetScript("OnClick", function() AddTimer("sv", JSHB.options.sv) end)

optsv.text1 = optsv:CreateFontString("JSBOptions_SVText1", "ARTWORK", optsv)
optsv.text1:SetFontObject(GameFontNormal)	
optsv.text1:SetJustifyH("LEFT")
optsv.text1:SetWidth(360)
optsv.text1:SetPoint("TOPLEFT", optsv.button1, "TOPLEFT", 0, -35)

JSHB.options.svscrollframe:SetScrollChild(optsv)
InterfaceOptions_AddCategory(JSHB.options.svscrollframeparent)

